"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toXlf1 = exports.toXlf2 = exports.fromXlf1 = exports.fromXlf2 = void 0;
var xmldoc_1 = require("xmldoc");
var translationFileModels_1 = require("./translationFileModels");
var XML_DECLARATION_MATCHER = /^<\?xml [^>]*>\s*/i;
function fromXlf2(xlf2) {
    var _a;
    var xmlDeclaration = (_a = xlf2.match(XML_DECLARATION_MATCHER)) === null || _a === void 0 ? void 0 : _a[0];
    var doc = new xmldoc_1.XmlDocument(xlf2);
    var file = doc.childNamed('file');
    var units = file.children
        .filter(function (n) { return n.type === 'element'; })
        .map(function (unit) {
        var _a, _b, _c, _d;
        var segment = unit.childNamed('segment');
        var notes = unit.childNamed('notes');
        return {
            id: unit.attr.id,
            source: toString.apply(void 0, segment.childNamed('source').children),
            target: toStringOrUndefined((_a = segment.childNamed('target')) === null || _a === void 0 ? void 0 : _a.children),
            state: segment.attr.state,
            meaning: toStringOrUndefined((_b = notes === null || notes === void 0 ? void 0 : notes.childWithAttribute('category', 'meaning')) === null || _b === void 0 ? void 0 : _b.children),
            description: toStringOrUndefined((_c = notes === null || notes === void 0 ? void 0 : notes.childWithAttribute('category', 'description')) === null || _c === void 0 ? void 0 : _c.children),
            locations: (_d = notes === null || notes === void 0 ? void 0 : notes.children.filter(function (n) { return n.type === 'element' && n.attr.category === 'location'; }).map(function (note) {
                var _a = note.val.split(':', 2), file = _a[0], lines = _a[1];
                var _b = lines.split(',', 2), lineStart = _b[0], lineEnd = _b[1];
                return {
                    file: file,
                    lineStart: parseInt(lineStart, 10),
                    lineEnd: lineEnd !== undefined ? parseInt(lineEnd, 10) : undefined
                };
            })) !== null && _d !== void 0 ? _d : []
        };
    });
    return new translationFileModels_1.TranslationFile(units, doc.attr.srcLang, doc.attr.trgLang, xmlDeclaration);
}
exports.fromXlf2 = fromXlf2;
function fromXlf1(xlf1) {
    var _a;
    var xmlDeclaration = (_a = xlf1.match(XML_DECLARATION_MATCHER)) === null || _a === void 0 ? void 0 : _a[0];
    var doc = new xmldoc_1.XmlDocument(xlf1);
    var file = doc.childNamed('file');
    var units = file.childNamed('body').children
        .filter(function (n) { return n.type === 'element'; })
        .map(function (unit) {
        var _a, _b, _c;
        var notes = unit.childrenNamed('note');
        var target = unit.childNamed('target');
        return {
            id: unit.attr.id,
            source: toString.apply(void 0, unit.childNamed('source').children),
            target: toStringOrUndefined(target === null || target === void 0 ? void 0 : target.children),
            state: target === null || target === void 0 ? void 0 : target.attr.state,
            meaning: toStringOrUndefined((_a = notes === null || notes === void 0 ? void 0 : notes.find(function (note) { return note.attr.from === 'meaning'; })) === null || _a === void 0 ? void 0 : _a.children),
            description: toStringOrUndefined((_b = notes === null || notes === void 0 ? void 0 : notes.find(function (note) { return note.attr.from === 'description'; })) === null || _b === void 0 ? void 0 : _b.children),
            locations: (_c = unit.childrenNamed('context-group')
                .map(function (contextGroup) { return ({
                file: contextGroup.childWithAttribute('context-type', 'sourcefile').val,
                lineStart: parseInt(contextGroup.childWithAttribute('context-type', 'linenumber').val, 10)
            }); })) !== null && _c !== void 0 ? _c : []
        };
    });
    return new translationFileModels_1.TranslationFile(units, file.attr['source-language'], file.attr['target-language'], xmlDeclaration);
}
exports.fromXlf1 = fromXlf1;
function toString() {
    var nodes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        nodes[_i] = arguments[_i];
    }
    return nodes.map(function (n) { return n.toString({ preserveWhitespace: true, compressed: true }); }).join('');
}
function toStringOrUndefined(nodes) {
    return nodes ? toString.apply(void 0, nodes) : undefined;
}
function toXlf2(translationFile, options) {
    var _a;
    var doc = new xmldoc_1.XmlDocument("<xliff version=\"2.0\" xmlns=\"urn:oasis:names:tc:xliff:document:2.0\" srcLang=\"".concat(translationFile.sourceLang, "\">\n    <file id=\"ngi18n\" original=\"ng.template\">\n    </file>\n</xliff>"));
    if (translationFile.targetLang) {
        doc.attr.trgLang = translationFile.targetLang;
    }
    var file = doc.childNamed('file');
    file.children = translationFile.units.map(function (unit) {
        var _a;
        var u = new xmldoc_1.XmlDocument("<unit id=\"".concat(unit.id, "\"><segment><source>").concat(unit.source, "</source></segment></unit>"));
        var segment = u.childNamed('segment');
        if (unit.target !== undefined) {
            segment.children.push(new xmldoc_1.XmlDocument("<target>".concat(unit.target, "</target>")));
        }
        if (unit.state) {
            segment.attr.state = unit.state;
        }
        if (unit.meaning !== undefined || unit.description !== undefined || unit.locations.length) {
            var notes = new xmldoc_1.XmlDocument('<notes></notes>');
            u.children.splice(0, 0, notes);
            (_a = notes.children).push.apply(_a, unit.locations.map(function (location) { return new xmldoc_1.XmlDocument("<note category=\"location\">".concat(location.file, ":").concat(location.lineStart).concat(location.lineEnd ? ',' + location.lineEnd : '', "</note>")); }));
            if (unit.description !== undefined) {
                notes.children.push(new xmldoc_1.XmlDocument("<note category=\"description\">".concat(unit.description, "</note>")));
            }
            if (unit.meaning !== undefined) {
                notes.children.push(new xmldoc_1.XmlDocument("<note category=\"meaning\">".concat(unit.meaning, "</note>")));
            }
        }
        updateFirstAndLastChild(u);
        return u;
    });
    updateFirstAndLastChild(doc);
    return ((_a = translationFile.xmlHeader) !== null && _a !== void 0 ? _a : '') + pretty(doc, options);
}
exports.toXlf2 = toXlf2;
function toXlf1(translationFile, options) {
    var _a;
    var doc = new xmldoc_1.XmlDocument("<xliff version=\"1.2\" xmlns=\"urn:oasis:names:tc:xliff:document:1.2\">\n    <file source-language=\"".concat(translationFile.sourceLang, "\"  datatype=\"plaintext\" original=\"ng2.template\">\n    <body></body>\n    </file>\n</xliff>"));
    var file = doc.childNamed('file');
    if (translationFile.targetLang !== undefined) {
        // assure "correct" order:
        file.attr = {
            'source-language': translationFile.sourceLang,
            'target-language': translationFile.targetLang,
            datatype: 'plaintext',
            original: 'ng2.template'
        };
    }
    var body = file.childNamed('body');
    body.children = translationFile.units.map(function (unit) {
        var _a;
        var transUnit = new xmldoc_1.XmlDocument("<trans-unit id=\"".concat(unit.id, "\" datatype=\"html\">\n        <source>").concat(unit.source, "</source>\n</trans-unit>"));
        if (unit.target !== undefined) {
            var target = new xmldoc_1.XmlDocument("<target>".concat(unit.target, "</target>"));
            if (unit.state !== undefined) {
                target.attr.state = unit.state;
            }
            transUnit.children.push(target);
        }
        if (unit.description !== undefined) {
            transUnit.children.push(new xmldoc_1.XmlDocument("<note priority=\"1\" from=\"description\">".concat(unit.description, "</note>")));
        }
        if (unit.meaning !== undefined) {
            transUnit.children.push(new xmldoc_1.XmlDocument("<note priority=\"1\" from=\"meaning\">".concat(unit.meaning, "</note>")));
        }
        if (unit.locations.length) {
            (_a = transUnit.children).push.apply(_a, unit.locations.map(function (location) { return new xmldoc_1.XmlDocument("<context-group purpose=\"location\">\n            <context context-type=\"sourcefile\">".concat(location.file, "</context>\n            <context context-type=\"linenumber\">").concat(location.lineStart, "</context>\n        </context-group>")); }));
        }
        updateFirstAndLastChild(body);
        return transUnit;
    });
    return ((_a = translationFile.xmlHeader) !== null && _a !== void 0 ? _a : '') + pretty(doc, options);
}
exports.toXlf1 = toXlf1;
function updateFirstAndLastChild(u) {
    u.firstChild = u.children[0];
    u.lastChild = u.children[u.children.length - 1];
}
function isWhiteSpace(node) {
    return node.type === 'text' && !!node.text.match(/^\s*$/);
}
function isSourceOrTarget(node) {
    return node.name === 'source' || node.name === 'target';
}
/// removes all whitespace text nodes that are not mixed with other nodes. For source/target nodes whitespace is unchanged.
function removeWhitespace(node) {
    if (node.type === 'element' && isSourceOrTarget(node)) {
        return;
    }
    if (node.type === 'element' && node.children.every(function (n) { return n.type !== 'text' || isWhiteSpace(n); })) {
        node.children = node.children.filter(function (c) { return !isWhiteSpace(c); });
        updateFirstAndLastChild(node);
    }
    node.children.filter(function (n) { return n.type === 'element'; }).forEach(function (e) { return removeWhitespace(e); });
}
/// format with 2 spaces indentation, except for source/target nodes: there nested nodes are assured to keep (non-)whitespaces (potentially collapsed/expanded)
function pretty(doc, options) {
    removeWhitespace(doc);
    addPrettyWhitespace(doc, 0, options);
    return doc.toString({ preserveWhitespace: true, compressed: true });
}
function indentChildren(doc, indent) {
    for (var i = doc.children.length - 1; i >= 0; i--) {
        doc.children.splice(i, 0, new xmldoc_1.XmlTextNode('\n' + '  '.repeat(indent + 1)));
    }
    doc.children.push(new xmldoc_1.XmlTextNode('\n' + '  '.repeat(indent)));
    updateFirstAndLastChild(doc);
}
function addPrettyWhitespace(doc, indent, options, sourceOrTarget) {
    if (sourceOrTarget === void 0) { sourceOrTarget = false; }
    if (isSourceOrTarget(doc) || sourceOrTarget) {
        if (options.prettyNestedTags && doc.children.length && doc.children.every(function (c) { return isWhiteSpace(c) || c.type === 'element'; })) {
            doc.children = doc.children.filter(function (c) { return !isWhiteSpace(c); });
            updateFirstAndLastChild(doc);
            indentChildren(doc, indent);
            doc.children.forEach(function (c) { return c.type === 'element' ? addPrettyWhitespace(c, indent + 1, options, true) : null; });
        }
        return;
    }
    if (doc.children.length && doc.children.some(function (e) { return e.type === 'element'; })) {
        indentChildren(doc, indent);
        doc.children.forEach(function (c) { return c.type === 'element' ? addPrettyWhitespace(c, indent + 1, options) : null; });
    }
}
